#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Elections et cartes électorales - énoncé
# 
# D'après wikipédia, le [Gerrymandering](https://fr.wikipedia.org/wiki/Gerrymandering) est un terme politique nord-américain pour désigner le découpage des circonscriptions électorales ayant pour objectif de donner l’avantage à un parti, un candidat, ou un groupe donné. Et c'est ce que nous allons faire dans cette séance. C'est un problème tout-à-fait d'actualité : [Primaire de la droite : 10 228 bureaux de vote stratégiquement répartis](http://www.lemonde.fr/election-presidentielle-2017/article/2016/09/30/primaire-de-la-droite-10-228-bureaux-de-vote-strategiquement-repartis_5005831_4854003.html).

# In[1]:


get_ipython().run_line_magic('matplotlib', 'inline')


# In[2]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Données
# 
# Les données sont de plusieurs types et sont regroupées en trois fichiers :
# 
# * [Résultat des élections législatives françaises de 2012 au niveau bureau de vote](https://www.data.gouv.fr/fr/datasets/resultat-des-elections-legislatives-francaises-de-2012-au-niveau-bureau-de-vote-nd/)
# * [Countours des circonscriptions des législatives](https://www.data.gouv.fr/fr/datasets/countours-des-circonscriptions-des-legislatives-nd/)
# * [Localisation des buraux de votes](http://www.xavierdupre.fr/enseignement/complements/bureauxvotegeo.zip)
# * [Localisation des villes](https://github.com/sdpython/actuariat_python/raw/master/src/actuariat_python/data/data_elections/villes_geo.zip)
# * [Localisation des bureaux de vote avec Cartelec](http://www.cartelec.net/?page_id=3609), cette base requiert la conversion des coordonnées (lire ce [notebook](http://www.xavierdupre.fr/app/ensae_teaching_cs/helpsphinx/notebooks/td1a_cenonce_session_12.html#cartographie))
# 
# Il est conseillé de télécharger directement ces données. Les paragraphes suivants expliquent comment ceux-ci ont été récupérés ou construit. Il n'est pas immédiat d'obtenir la localisation des bureaux de vote. Celle-ci n'est d'ailleurs pas complète.

# ### Résultats des élections législatives

# In[3]:


from actuariat_python.data import elections_legislatives_bureau_vote
tour = elections_legislatives_bureau_vote(source='xd')


# In[4]:


tour["T2"].sort_values(["Code département", "N° de circonscription Lg"]).head()


# ### Géolocalisation des circonscription

# In[5]:


from actuariat_python.data import elections_legislatives_circonscription_geo
geo = elections_legislatives_circonscription_geo()


# In[6]:


geo.sort_values(["department", "code_circonscription"]).head()


# In[7]:


c = list(geo.sort_values(["department", "code_circonscription"])["communes"])[0].split("-")
c.sort()
c[:5]


# In[8]:


list(geo.sort_values(["department", "code_circonscription"])["kml_shape"])[:1]


# ### Géolocation des bureaux de vote
# 
# Ces données sont importantes afin de pouvoir associer chaque bureau à une circonscription. C'est la donnée la plus compliquée à obtenir car elle nécessite de combiner plusieurs techniques et sources de données pour obtenir une table propre. Le fichier final peut être obtenu comme suit :

# In[9]:


from actuariat_python.data import elections_vote_places_geo
bureau_geo = elections_vote_places_geo()


# In[10]:


bureau_geo.head()


# **Ce qui suit explique la façon dont j'ai constuit cette table.**
# 
# Les bureaux sont assez stables d'une élection à l'autre et cela ne devrait pas trop poser de problèmes si on mélange les données. En revanche, ces données sont assez difficiles à obtenir. [open.data.fr](https://www.data.gouv.fr/fr/datasets/?q=bureaux+de+vote&page=1) propose ces données mais il faut récupérer chaque ville ou chaque région séparément sans garantie de réussir à couvrir tout le territoire. Le site [NosDonnes.fr](http://www.nosdonnees.fr/dataset/golocalisation-des-bureaux-de-votes) recense bien toutes ces informations mais il n'est pas possible - au moment où j'écris ces lignes - de récupérer ces données sous la forme d'un fichier plat. De plus, certaines régions ne sont disponibles que sous forme de scan d'impressions papier. C'est en lisant l'article [Comment redécoupe-t-on la carte électorale?](http://www.slate.fr/story/3945/comment-redecouper-t-la-carte-electorale) que je suis tombé finalement sur la base constituée pour rédiger l'article [Etude sur le redécoupage électoral : évaluer le poids politique de la réforme](https://www.regardscitoyens.org/etude-sur-le-redecoupage-electoral-poids-politique-reforme/). Les données sont accessibles sur [RegardsCitoyens.fr](https://www.regardscitoyens.org/publication/#donnees) et en cherchant bien, on trouve le répertoire [redécoupage](https://www.regardscitoyens.org/telechargement/redecoupage/) et le fichier [2014041401_resultats.csv.zip](https://www.regardscitoyens.org/telechargement/redecoupage/2014041401_resultats.csv.zip). Cependant, la géolocalisation des bureaux n'est pas souvent renseignée. On peut se contenter des fichiers obtenus pour quelques zones seulement : [Paris](https://www.data.gouv.fr/fr/datasets/liste-des-bureaux-de-vote-prs/), [Gironde](https://www.data.gouv.fr/fr/datasets/geo-localisation-des-bureaux-de-vote-du-departement-de-la-gironde-rdl/), [Montpellier](https://www.data.gouv.fr/fr/datasets/bureaux-de-vote-mtn/), [Marseille](https://www.data.gouv.fr/fr/datasets/bureaux-de-vote-de-marseille-op/), [Saint-Malo](https://www.data.gouv.fr/fr/datasets/emplacement-des-bureaux-de-vote/), [Nogent-Sur-Marne](https://www.data.gouv.fr/fr/datasets/bureaux-de-vote-de-la-ville-de-nogent-sur-marne/), [Haut-de-Seine](https://www.data.gouv.fr/fr/datasets/bureaux-de-vote-7/), [Toulouse](https://www.data.gouv.fr/fr/datasets/bureaux-de-vote-2012/), [Grand-Poitiers](https://www.data.gouv.fr/fr/datasets/citoyennete-bureaux-de-vote-grand-poitiers-donnees-de-reference/), [Calvados](https://www.data.gouv.fr/fr/datasets/bureaux-centralisateurs-des-circonscriptions-cantonales-dans-le-calvados/). Cette approche risque d'être fastidieuse dans la mesure où les formats pour chaque ville ont de grande chance d'être différent. 
# 
# La meilleure option est peut-être de scrapper le site [bureaudevote.fr](http://bureaudevote.fr/bureaudevote00.htm) - qui fonctionne bien quand il n'est pas en maintenance - en espérant que les numéros des bureaux de votes correspondent. Le site propose seulement les adresses. Il faudra utiliser une API de geocoding. Quelque soit la source, on supposera alors que tous les bureaux de vote associés au même emplacement feront nécessairement partie de la même circonscription. Bref, **l'open data passe aussi par une certaine standardisation !**
# 
# Certains départements ne sont pas renseignés, la Drôme par exemple. Il faut aller sur d'autres sites comme [linternaute.com](http://www.linternaute.com/ville/valence/ville-26362/bureaux-vote#ville-26362-1), accéder à la [carte](http://www.linternaute.com/ville/p-bureau-vote#trouver-bureau-vote), mais il faudra un peu plus de temps pour récupérer toutes ces informations. Le code proposé ci-dessus récupère les coordonnées des bureaux de vote. Cela prend plusieurs heures et il faut relancer le processus quand il s'arrête pour une erreur de réseau. Il est conseillé de télécharger le résultat. Le géocodeur d'[OpenStreetMap](http://openstreetmap.fr/) n'est pas de très bonne qualité sur la France. Il ne retourne rien dans plus d'un tiers des cas. On peut compléter avec l'[API de Bing Maps](https://www.microsoft.com/maps/choose-your-bing-maps-API.aspx).

# In[11]:


from actuariat_python.data import elections_vote_place_address
bureau = elections_vote_place_address(hide_warning=True)


# In[12]:


bureau.head()


# On récupère une clé pour utiliser l'[API de Bing Maps](https://www.microsoft.com/maps/choose-your-bing-maps-API.aspx) avec le module [keyring](http://pythonhosted.org/keyring/). Pour stocker son mot de passe sur la machine, il suffit d'écrire :
# 
# ```
# import keyring
# keyring.get_password("bing", "actuariat_python,key")
# ```

# In[13]:


import keyring, os
bing_key = keyring.get_password("bing", "actuariat_python,key")
coders = ["Nominatim"]
if bing_key:
    # si la clé a été trouvée
    coders.append(("bing", bing_key))
len(coders)


# In[14]:


import os
if not os.path.exists("bureauxvotegeo.zip"):
    from actuariat_python.data import geocode
    from pyquickhelper.loghelper import fLOG
    fLOG(OutputPrint=True)
    bureau_geo = geocode(bureau, fLOG=fLOG, index=False, encoding="utf-8",
                         exc=False, save_every="bureau.dump.txt", sep="\t", every=100,
                         coders=coders)
else:
    print("Les données ont déjà été geocodées.")


# On regarde les valeurs manquantes.

# In[15]:


import missingno
missingno.matrix(bureau_geo, figsize=(12, 6));


# On pourra finalement récupérer la base des géocodes comme ceci :

# In[16]:


from actuariat_python.data import elections_vote_places_geo
places = elections_vote_places_geo()
places.head()


# ### Géolocalisation des villes

# In[17]:


from actuariat_python.data import elections_vote_places_geo
bureau_geo = elections_vote_places_geo()
villes_geo = bureau_geo[["city", "zip", "n"]].groupby(["city", "zip"], as_index=False).count()
villes_geo.head()


# In[18]:


from actuariat_python.data import villes_geo
villes_geo = villes_geo(as_df=True)
villes_geo.head()


# In[19]:


import keyring, os
bing_key = keyring.get_password("bing", "actuariat_python,key")
coders = []
if bing_key:
    # si la clé a été trouvée
    coders.append(("bing", bing_key))
len(coders)


# In[20]:


import os
geocode = True
if geocode:
    if os.path.exists("villes_geo.txt"):
        import pandas
        villes_geo = pandas.read_csv("villes_geo.txt", sep="\t", encoding="utf-8")
    else:
        from actuariat_python.data import geocode
        from pyquickhelper.loghelper import fLOG
        fLOG(OutputPrint=True)
        villes_geo = geocode(villes_geo, fLOG=fLOG, index=False, encoding="utf-8",
                             exc=False, save_every="villes.dump.txt", sep="\t", every=100,
                             coders=coders, country="France")


# In[21]:


villes_geo.head()


# On conserve les données pour éviter de les reconstuire et faire appel à l'API Bing à nouveau.

# In[22]:


villes_geo.to_csv("villes_geo.txt", sep="\t", index=False, encoding="utf-8")


# In[23]:


villes_geo.shape


# ## Géolocation des bureaux de vote avec Cartélec
# 
# Le site [cartelec](http://www.cartelec.net/?page_id=3609) recense beaucoup plus de bureaux de vote mais pour les élections 2007. Ils ne devraient pas avoir changé beaucoup.

# In[24]:


from pyensae.datasource import download_data
shp_vote = download_data("base_cartelec_2007_2010.zip")
shp_vote


# In[25]:


# La version 2.0.0.dev de pyshp est buggée. Il vaut mieux ne pas l'utiliser.
import shapefile
if "dev" in shapefile.__version__:
    raise ImportError("Use a different version of pyshp not '{0}'".format(shapefile.__version__))
r = shapefile.Reader("fond0710.shp", encoding="utf8", encodingErrors="ignore")
shapes = r.shapes()
records = r.records()
len(shapes), len(records)


# In[26]:


{k[0]: v for k, v in zip(r.fields, records[0])}


# In[27]:


shapes[0].points


# ## Exercice 1
# 
# * Etablir un plan d'action
# * Détailler la mise en oeuvre de ce plan à partir des données
# * Répartir les tâches sur plusieurs équipes

# ## Exercice 2
# 
# Mettre en oeuvre le plan d'action.

# In[28]: